# coding: utf-8
"""Usage:
  ta-report [<report_type>... --work-dir <PATH> --user <NAME> --zip --clean --dumps-src -d]

Options:
  -h --help             Show this message
  -V --version          Show version
  -w --work-dir <PATH>  Path to sources
  -u --user <NAME>      Set user for logging
  -z --zip              Compress to `result.zip`
  -c --clean            Delete created files except result
  --dumps-src           Try to save merged sources into xlsx or tsv:
                          <report_type>_dumps-src.{xlsx|tsv}
  -d --debug            Dump log into the local file `ta-report.log`

Note:
  <report_type>  report, search_queries
"""
import logging
import traceback
import warnings
import os
import sys
import matplotlib
matplotlib.use('TkAgg')
import seaborn as sns  # noqa: 402 (module top level)
from docopt import docopt  # noqa: 402 (module top level)
from .const import (__version__, __PIP_PACKAGE_NAME__, SESSION_ID, CLI_REPORTS)  # noqa: 402 (module top level)
from .logger import make_logger, LOG_EXTRA  # noqa: 402 (module top level)
# REPORTS is Dispatcher
from .reports import REPORTS  # noqa: 402 (module top level)
from .utils import check_pkg_version, colorize_str_to_console  # noqa: 402 (module top level)

sns.plt.rcParams["figure.figsize"] = [15, 10]


def main():
    """() -> None; CLI Entry Point"""
    try:
        ARGS = docopt(__doc__, version=__version__)
        print("session_id: %s" % SESSION_ID)
        if '--work-dir' in ARGS and ARGS['--work-dir']:
            os.chdir(ARGS['--work-dir'])
            LOG_EXTRA["app_workdir"] = ARGS['--work-dir']
        if '--user' in ARGS and ARGS['--user']:
            LOG_EXTRA["app_user"] = ARGS['--user']
        # if empty do all
        if "<report_type>" in ARGS and not ARGS["<report_type>"]:
            ARGS["<report_type>"] = CLI_REPORTS
        for fn in ARGS["<report_type>"]:
            if fn not in CLI_REPORTS:
                err = KeyError("unknown report type `%s`. Valid options: %s" % (fn, CLI_REPORTS))
                print(colorize_str_to_console(err, "red"))
                sys.exit(1)
            else:
                LOG_EXTRA["app_reports"].append(fn)
        LOG_EXTRA["app_args"] = ARGS
        log = make_logger(LOG_EXTRA, is_debug=ARGS["--debug"])
        warnings.filterwarnings("ignore")
        print("START APP")
        log(logging.INFO, "START APP", "app_start")
        check_pkg_version(log, __PIP_PACKAGE_NAME__, __version__)
        for fn_name, fn in REPORTS.items():
            if fn_name not in LOG_EXTRA["app_reports"]:
                continue
            report = fn(log)
            if ARGS['--dumps-src'] and report is not None:
                if report.shape[0] < 500000:  # ограничение с потолка
                    assert report.shape[0] < 1000000, "1000000 is max for xlsx"
                    report.to_excel("./%s_dumps-src.xlsx" % fn_name, encoding="utf-8", sheet_name="data")
                report.to_csv(
                    "./%s_dumps-src.tsv" % fn_name,
                    sep='\t',
                    encoding="utf-8",
                    chunksize=100000)

        if ARGS['--zip']:
            import zipfile
            import shutil
            print 'creating archive'
            log(logging.INFO, "create archive", "app_info")
            try:
                ext_name = 'extras'  # extra data
                shutil.make_archive(ext_name, 'zip', base_dir='images')
                log(logging.INFO, "add `images` to %s" % ext_name, "app_info")
                zip_name = 'reports.zip'  # minimum
                zf = zipfile.ZipFile(zip_name, mode='w')
                for report in LOG_EXTRA["app_reports"]:
                    report_file = "%s.xlsx" % report
                    if not os.path.exists(report_file):
                        continue
                    zf.write(report_file)
                    os.remove(report_file)
                    log(logging.INFO, "add file to %s" % zip_name, "app_info")
                zf.close()
            except:
                raise
        if ARGS['--clean']:
            import shutil
            for x in ["charts", "images", "tmp_data"]:
                shutil.rmtree(x, ignore_errors=True)
                log(logging.INFO, "remove (--clean): %s" % x, "app_info")
        print("FINISH APP")
        log(logging.INFO, "FINISH APP", "app_finish")
        sys.exit(0)
    except Exception as err:
        print(colorize_str_to_console(err, "red"))
        with open('stderr.txt', 'w') as f:
            f.write("SessionId: %s\n" % SESSION_ID)
            traceback.print_exc(file=f)
        log(logging.ERROR, err, "app_error", exc_info=1)
        sys.exit(1)


if __name__ == "__main__":
    main()
