# coding: utf-8
"""utils"""
import re
import logging
from glob import glob
import tarfile as tar
from os.path import isfile, splitext, basename
from .xlsx_search_queries.new_ctr import new_ctr_col
from .xlsx_search_queries.new_cpc import new_cpc_col
from .utils import (
    colorize_str_to_console,
    extract_currency_from_tar,
    extract_str_currency_from_tar,
    init_app
)
import pandas as pd


def process_login(name):
    "(str) -> str"

    return name.replace('.', '-').lower().strip()


def extract_login(filename):
    "(str) -> str"

    return process_login(splitext(basename(filename))[0].split("_", 1)[0])


def sources_from_tar(logger, report_suffix):
    sources = glob("./*.tar.gz")
    currency = reduce(
        lambda x, y: x if x == y else None, map(extract_str_currency_from_tar, sources)
    )
    has_equal_currency = currency is not None or len(sources) == 1
    if not sources:
        raise IOError("sorces do not exist")
    init_app()

    def is_report(members, suffix):
        for tarinfo in members:
            if splitext(tarinfo.path)[0].endswith(suffix):
                yield tarinfo

    def extract_reports(tarpath):
        with tar.open(tarpath, "r:gz") as src:
            dfs = []
            for report in is_report(src, report_suffix):
                try:
                    df = pd.read_table(
                        src.extractfile(report),
                        quoting=3,  # None
                        encoding="utf-8",
                        low_memory=True,
                        engine="c")
                except pd.io.common.EmptyDataError:
                    msg = "file is empty: %s" % report.path
                    logger(logging.ERROR, msg, "app_error", exc_info=1)
                    print(colorize_str_to_console("empty file: {}".format(report.path), "red"))
                    continue
                try:
                    df["Currency"] = extract_currency_from_tar(tarpath)
                except ValueError:
                    if has_equal_currency:
                        df["Currency"] = 1.0
                    else:
                        raise
                if df.shape[0] > 0:
                    dfs.append(df)
                else:
                    msg = "tar is empty: %s" % tarpath
                    logger(logging.ERROR, msg, "app_error", exc_info=1)
                    print(colorize_str_to_console("empty: {}".format(tarpath), "red"))
            if dfs:
                return pd.concat(dfs)
            else:
                None

    data = filter(lambda x: x is not None, map(extract_reports, sources))
    if not data:
        return None

    print("\nSources from `*%s.*`:" % report_suffix)
    for x in sources:
        print(x)

    msg = "\nsources from `*%s.*`: %s" % (report_suffix, sources)
    logger(logging.INFO, msg, "app_info")

    print("{:-<15}".format(""))
    print("files: {}".format(len(sources)))

    df = pd.concat(data)
    print("{:-<15}".format(""))
    msg = "read rows: {}".format(df.shape[0])
    print(msg)
    logger(logging.INFO, msg, "app_info")

    return df


def filter_by_campaigns(logger, df):

    if isfile("./campaigns.txt"):
        try:
            with open("./campaigns.txt", "rU") as src:
                cid_filter = src.readlines()
                cid_filter = {int(x.strip()) for x in cid_filter if x not in {"", "\n"}}
            if cid_filter:
                print("data has been filtered by ./campaigns.txt")
                return df.loc[df.CampaignID.isin(cid_filter)]
        except Exception as err:
            msg = "file ./campaigns.txt exists but something went wrong :("
            print('\n'.join([colorize_str_to_console(msg, "red"), err.message]))
            logger(logging.ERROR, msg, "app_error", exc_info=1)
    else:
        return df


def rename_report(logger, df):
    if df is None or df.shape[0] == 0:
        return None

    columns = {
        # Dimensions
        u"Currency": u"Currency",
        u"Логин": "LoginName",
        u"Название кампании": "CampaignName",
        u"ID кампании": "CampaignID",
        u"Название группы": "GroupName",
        u"ID группы": "GroupID",
        u"ID баннера": "DirectBannerID",
        u"Заголовок баннера": "Title",
        u"текст баннера": "Body",
        u"URL баннера": "URL",
        u"ID ключевого слова (direct)": "DirectPhraseID",
        u"ID ключевого слова (BS)": "BSPhraseID",
        u"Ключевое слово (без минус-слов)": "PhraseText",
        u"Все минус-слова, действующие для фразы (собственные и на кампанию), через пробел": "AntiWords",
        u"Количество слов из ключевого слова, входящих в заголовок и текст баннера": "BoldedWordsCnt",
        u"Количество слов в ключевом слове": "WordsCnt",
        u"Список слов фразы, не входящих в баннер (через пробел)": "NonBoldedWords",
        u"Хеш картинки баннера": "ImageHash",
        u"количество сайтлинков": "SitelinksCnt",
        u"флаг наличия шаблона в баннере": "HasTemplate",
        u"Флаг корректного использования шаблона (при подстановке текущего ключевого слова баннер соответствует ограничениям директа)": \
        u"IsCorrectTemplate",
        u"Количество уточнений": "CalloutsCnt",  # Уточнения https://yandex.ru/support/direct/features/callout.xml?lang=en
        u"Cредняя позиция показов (spec)": "AvgPosShowsSpec",
        u"cредняя позиция кликов (spec)": "AvgPosClicksSpec",
        u"Cредняя позиция показов (others)": "AvgPosShowsOther",
        u"cредняя позиция кликов (others)": "AvgPosClicksOther",
        u"Продуктивность фразы": "Productivity",
        u"текущая ставка на поиске": "BidSearch",
        u"текущая ставка на тематических площадках": "BidRSYA",
        u"дневной лимит на кампанию": "DayBudget",
        u"код временного таргетинга": "TimeTarget",
        u"Стратегия показа": "Strategy",
        # Dimensions added 26.12.2016
        u"Использование шаблона в отображаемой ссылке": "DisplayUrlHasTemplate",
        u'количество описаний сайтлинков': "DescriptionSLCnt",
        u'Заполненность визитки': "VisitCardUsage",
        # Mobile Dimensions added 10.01.2017
        u'Мобильное объявление': "isMobileBanner",
        # Dimensions added 19.01.2017
        u'Тип объявления': "AdType",
        u'Подтип объявления': "AdSubtype",
        u'Размер изображения (для фикс. размеров)': "FixedImageSize",
        # Mobile Metrics added 10.01.2017
        u'Показы на мобильных': "ShowsMobile",
        u'Клики на мобильных': "ClicksMobile",
        u'Расходы на мобильных': "CostMobile",
        # Metrics
        u"Показы на поиске в блоке спецразмещения": "ShowsSpec",
        u"Клики на поиске в блоке спецразмещения": "ClicksSpec",
        u"Расходы на поиске за клики в блоке спецразмещения": "CostSpec",
        u"Показы на поиске в блоках гарантированных и динамических показов": "ShowsOther",
        u"Клики на поиске в блоках гарантированных и динамических показов": "ClicksOther",
        u"Расходы на поиске за клики в блоке гарантированных и динамических показов": "CostOther",
        u"Показы на тематических площадках": "ShowsRSYA",
        u"Клики на тематических площадках": "ClicksRSYA",
        u"Расходы за клики на тематических площадках": "CostRSYA",
    }

    df.rename(columns=columns, inplace=True)
    try:
        assert all(df.columns.isin(columns.values())), u"Some columns hasn't been translated"
    except AssertionError as err:
        msg = "Missed: %s" % df.columns[~df.columns.isin(columns.values())]
        err.message = "\n".join([err.message, msg])
        logger(logging.ERROR, msg, "app_error", exc_info=1)
        raise err

    return df


def preprocess_report(logger, df):

    if df is None or df.shape[0] == 0:
        return None

    df = rename_report(logger, df)

    df["Shows"] = df.ShowsOther + df.ShowsRSYA + df.ShowsSpec
    df["Clicks"] = df.ClicksOther + df.ClicksRSYA + df.ClicksSpec
    df["Cost"] = df.CostOther + df.CostRSYA + df.CostSpec
    df["ShowsSearch"] = df.ShowsOther + df.ShowsSpec
    df["ClicksSearch"] = df.ClicksOther + df.ClicksSpec
    df["CostSearch"] = df.CostOther + df.CostSpec

    if df.VisitCardUsage.dtype.name not in {"float32", "float64", "float128"}:
        df.VisitCardUsage.replace({"None": pd.np.nan}, inplace=True)
        df.VisitCardUsage = df.VisitCardUsage.astype(pd.np.float64)

    assert df.VisitCardUsage.dtype == 'float64'
    df.VisitCardUsage = df.VisitCardUsage.apply(lambda x: None if pd.isnull(x) else x > 0.95)

    if "BoldedWordsCnt" in df.columns and df.BoldedWordsCnt.dtype.name not in {"float64", "int64"}:
        df.BoldedWordsCnt = df.BoldedWordsCnt.replace({"None": pd.np.nan}).astype(pd.np.float64)
    # todo(n-bar) remove it if CUSTOMSOLUTIONS-1950 is closed
    if "BoldedWordsCnt" in df.columns and "WordsCnt" in df.columns:
        fix_cnt = lambda row: min(row["BoldedWordsCnt"], row["WordsCnt"])  # noqa: E731
        df["BoldedWordsCnt"] = df.apply(fix_cnt, axis=1)

    if df.DisplayUrlHasTemplate.dtype != 'bool':
        df.DisplayUrlHasTemplate = df.DisplayUrlHasTemplate.replace({"None": None, "True": True, "False": False})

    def contain_template(row):

        return '#' in " ".join([row["Title"], row["Body"]])

    def is_correct_template(row):
        if row["IsCorrectTemplate"] is None or not contain_template(row):
            return None

        return row["IsCorrectTemplate"] and contain_template(row)

    if "IsCorrectTemplate" in df.columns:
        if df.IsCorrectTemplate.dtype != 'bool':
            df.IsCorrectTemplate = df.IsCorrectTemplate.replace({"None": None, "True": True, "False": False})
        # todo(n-bar) remove it if CUSTOMSOLUTIONS-1951 is closed
        # df["IsCorrectTemplate"] = df.apply(is_correct_template, axis=1)

    if "PhraseText" in df.columns:
        filtered_rows = df.PhraseText.isnull()
        if sum(filtered_rows) > 0:
            df = df.loc[~filtered_rows]
            msg = "filtered by `PhraseText is null`: %s" % sum(filtered_rows)
            print(msg)
            logger(logging.INFO, msg, "app_error")

    if "WordsCnt" in df.columns:
        filtered_rows = df.WordsCnt > 0
        if sum(~filtered_rows) > 0:
            df = df.loc[filtered_rows]
            msg = "filtered by `WordsCnt > 0`: %s" % sum(~filtered_rows)
            print(msg)
            logger(logging.INFO, msg, "app_error")

    if df.HasTemplate.dtype != 'bool':
        df.HasTemplate = df.HasTemplate.replace({"None": None, "True": True, "False": False})

    if "isMobileBanner" in df.columns and df.isMobileBanner.dtype != 'bool':
        df.isMobileBanner = df.isMobileBanner.replace({"None": None, "True": True, "False": False})

    # convert currency
    currency_cols = [
        "Cost",
        "CostSearch",
        "CostRSYA",
        "CostSpec",
        "CostOther",
        "CostMobile",
        "BidRSYA",
        "BidSearch"]

    for col in currency_cols:
        df[col] = df[col].replace('None', '0')
        df[col] = df[col].replace('', '0')
        df[col] = df[col].astype(pd.np.float64) * df.Currency

    df.drop(["Currency"], axis=1, inplace=True)
    df = filter_by_campaigns(logger, df)
    df["ShowsDesktop"] = df["Shows"] - df["ShowsMobile"]
    df["ClicksDesktop"] = df["Clicks"] - df["ClicksMobile"]
    df["CostDesktop"] = df["Cost"] - df["CostMobile"]

    return df


def preprocess_search_queries(logger, df):

    if df is None or df.shape[0] == 0:
        return None

    def split_text_by_comma(text):

        if text is pd.np.nan:
            text = u""

        if not isinstance(text, unicode):
            raise TypeError(u"Expect <type 'unicode'>, in fact: %s" % type(text))

        return [t.strip() for t in re.split(u", *", text) if t]

    columns = {
        u"Поисковый запрос": "Query",
        u"Кампания": "CampaignName",
        u"Кампания №": "CampaignID",
        u"Тип кампании": "CampaignType",
        u"Баннер №": "DirectBannerID",
        u"Условие показа": "PhraseText",
        u"Показы": "Shows",
        u"Клики": "Clicks",
        u"Расходы": "Cost",
        u"Текст заголовка": "Title",
        u"Текст тела": "Body",
        u"Тексты быстрых ссылок (через запятую)": "SLText",
        u'Количество слов в "Условие показа"': "PhraseWordsCnt",
        u'Количество слов в условии показа': "PhraseWordsCnt",
        u'Количество слов в "Поисковый запрос"': "QueryWordsCnt",
        u'Количество слов в поисковом запросе': "QueryWordsCnt",
        u'Подсветка для "Заголовок + тело"': "QueryHighlighTitleBody",
        u'Подсветка для заголовке + теле': "QueryHighlighTitleBody",
        u'Подсветка для "Заголовок + тело + быстрые ссылки"': "QueryHighlightTitleBodySL",
        u'Подсветка для заголовка + тела + быстрых ссылок': "QueryHighlightTitleBodySL",
        u'Слова запроса без подсветки "Заголовок + тело"': "QueryNonBoldedWordsTitleBody",
        u'Слова запроса без подсветки для заголовка + тела': "QueryNonBoldedWordsTitleBody",
        u'Слова запроса без подсветки "Заголовок + тело + быстрые ссылки"': "QueryNonBoldedWordsTitleBodySL",
        u'Слова запроса без подсветки  для заголовка + тела + быстрых ссылок': "QueryNonBoldedWordsTitleBodySL",
        u'Currency': "Currency",
        # added in CUSTOMSOLUTIONS-1965 2017-04-04
        u'Тип соответствия': "MatchType"  # NONE, RELATED_KEYWORD, SYNONYM
        u''
    }
    df.rename(columns=columns, inplace=True)
    try:
        assert all(df.columns.isin(columns.values())), "Some columns hasn't been translated"
    except AssertionError as err:
        msg = "Missed: %s" % df.columns[~df.columns.isin(columns.values())]
        err.message = "\n".join([err.message, msg])
        logger(logging.ERROR, msg, "app_error", exc_info=1)
        raise err

    # filters
    df = filter_by_campaigns(logger, df)
    filtered_rows = df.QueryWordsCnt < df.PhraseWordsCnt
    if sum(filtered_rows) > 0:
        msg = "filtered by `QueryWordsCnt < PhraseWordsCnt`: %s" % sum(filtered_rows)
        print(colorize_str_to_console(msg, "red"))
        logger(logging.INFO, msg, "app_error")

    df = df.loc[(df["Shows"] > 0) & ~filtered_rows]

    filtered_rows = (df.QueryWordsCnt == 0) | (df.PhraseWordsCnt == 0)
    if sum(filtered_rows) > 0:
        msg = "filtered by `(QueryWordsCnt == 0) | (PhraseWordsCnt == 0)`: %s" % sum(filtered_rows)
        print(colorize_str_to_console(msg, "red"))
        logger(logging.INFO, msg, "app_error")
    df = df.loc[~((df.QueryWordsCnt == 0) | (df.PhraseWordsCnt == 0))]

    # convert currency
    df["Cost"] = df["Cost"] * df["Currency"]
    df.drop(["Currency"], axis=1, inplace=True)
    # calculations
    df["QueryWordsCnt"] = df["QueryWordsCnt"].apply(lambda x: min(x, 7))
    df["PhraseWordsCnt"] = df["PhraseWordsCnt"].apply(lambda x: min(x, 7))
    df["QueryNonBoldedWordsTitleBody"] = df["QueryNonBoldedWordsTitleBody"].apply(lambda x: u" ".join(sorted(split_text_by_comma(x))))
    df["QueryNonBoldedWordsTitleBodySL"] = df["QueryNonBoldedWordsTitleBodySL"].apply(lambda x: u" ".join(sorted(split_text_by_comma(x))))
    df["NewCTRFromNested"] = new_ctr_col(df)
    df["NewCPCFromNested"] = new_cpc_col(df)
    df["NewClicksFromNested"] = (df["Shows"] * df["NewCTRFromNested"] - df["Clicks"]).apply(lambda val: max(0, val))
    df["NewCostFromNested"] = df["NewClicksFromNested"] * df["NewCPCFromNested"]

    return df


def preprocess_mobile(logger, df):
    """(DataFrame) -> DataFrame"""

    if df is None or df.shape[0] == 0:
        return None

    columns = {
        # Dimensions
        u"Currency": u"Currency",
        u"Логин": "LoginName",
        u"Название кампании": "CampaignName",
        u"ID кампании": "CampaignID",
        u"Название группы": "GroupName",
        u"ID группы": "GroupID",
        u"ID баннера": "DirectBannerID",
        u"Заголовок баннера": "Title",
        u"текст баннера": "Body",
        u"ID ключевого слова (direct)": "DirectPhraseID",
        u"ID ключевого слова (BS)": "BSPhraseID",
        u"Ключевое слово (без минус-слов)": "PhraseText",
        u"Количество слов из ключевого слова, входящих в заголовок и текст баннера": "BoldedWordsCnt",
        u"Количество слов в ключевом слове": "WordsCnt",
        u"Список слов фразы, не входящих в баннер (через пробел)": "NonBoldedWords",
        u"Хеш картинки баннера": "ImageHash",
        u"флаг наличия шаблона в баннере": "HasTemplate",
        u"Флаг корректного использования шаблона (при подстановке текущего ключевого слова баннер соответствует ограничениям директа)": \
        u"IsCorrectTemplate",
        u"Cредняя позиция показов (spec)": "AvgPosShowsSpec",
        u"cредняя позиция кликов (spec)": "AvgPosClicksSpec",
        u"Cредняя позиция показов (others)": "AvgPosShowsOther",
        u"cредняя позиция кликов (others)": "AvgPosClicksOther",
        u"текущая ставка на поиске": "BidSearch",
        u"текущая ставка на тематических площадках": "BidRSYA",
        u"дневной лимит на кампанию": "DayBudget",
        u"код временного таргетинга": "TimeTarget",
        u"Стратегия показа": "Strategy",
        u'Тип объявления': "AdType",
        u'Размер изображения (для фикс. размеров)': "FixedImageSize",
        # Mobile Metrics added 10.01.2017
        u'Показы на мобильных': "ShowsMobile",
        u'Клики на мобильных': "ClicksMobile",
        u'Расходы на мобильных': "CostMobile",
        # Metrics
        u"Показы на поиске в блоке спецразмещения": "ShowsSpec",
        u"Клики на поиске в блоке спецразмещения": "ClicksSpec",
        u"Расходы на поиске за клики в блоке спецразмещения": "CostSpec",
        u"Показы на поиске в блоках гарантированных и динамических показов": "ShowsOther",
        u"Клики на поиске в блоках гарантированных и динамических показов": "ClicksOther",
        u"Расходы на поиске за клики в блоке гарантированных и динамических показов": "CostOther",
        u"Показы на тематических площадках": "ShowsRSYA",
        u"Клики на тематических площадках": "ClicksRSYA",
        u"Расходы за клики на тематических площадках": "CostRSYA",
        # MobileApp Specific
        u"Ссылка на приложение": "AppLink",
        u"Возрастные ограничения": "AgeLimit",
        # Targeting
        u"Показывать на смартфонах": "ShowOnSmartphones",
        u"Показывать на планшетах": "ShowOnTablets",
        u"Показывать в мобильной сети": "ShowOnCellular",
        u"Тип ОС": "OSType",
        u"Версия ОС": "OSVersion",
        # Additional parameters
        u"Используется трекинговая ссылка": "HasTrackingLink",
        u"Отображать иконку": "ShowIcon",
        u"Отображать цену": "ShowPrice",
        u"Отображать рейтинг": "ShowRating",
        u"Отображать количество оценок": "ShowRatingsCount",
        # Metrics
        u"Количество установок": "Installations",
        u"Установки (mobile Andriod)": "InstallsMobileAndroid",
        u"Установки (mobile iOS)": "InstallsMobileIOS",
        u"Установки (tablet Andriod)": "InstallsTabletAndroid",
        u"Установки (tablet iOS)": "InstallsTabletiOS"
    }

    df.rename(columns=columns, inplace=True)
    try:
        assert all(df.columns.isin(columns.values())), u"Some columns hasn't been translated"
    except AssertionError as err:
        msg = "Missed: %s" % df.columns[~df.columns.isin(columns.values())]
        err.message = "\n".join([err.message, msg])
        logger(logging.ERROR, msg, "app_error", exc_info=1)
        raise err

    df["Shows"] = df.ShowsOther + df.ShowsRSYA + df.ShowsSpec
    df["Clicks"] = df.ClicksOther + df.ClicksRSYA + df.ClicksSpec
    df["Cost"] = df.CostOther + df.CostRSYA + df.CostSpec
    df["ShowsSearch"] = df.ShowsOther + df.ShowsSpec
    df["ClicksSearch"] = df.ClicksOther + df.ClicksSpec
    df["CostSearch"] = df.CostOther + df.CostSpec

    if "BoldedWordsCnt" in df.columns and df.BoldedWordsCnt.dtype.name not in {"float64", "int64"}:
        df.BoldedWordsCnt = df.BoldedWordsCnt.replace({"None": pd.np.nan}).astype(pd.np.float64)
    if "BoldedWordsCnt" in df.columns and "WordsCnt" in df.columns:
        fix_cnt = lambda row: min(row["BoldedWordsCnt"], row["WordsCnt"])  # noqa: E731
        df["BoldedWordsCnt"] = df.apply(fix_cnt, axis=1)

    if df.IsCorrectTemplate.dtype != 'bool':
        df.IsCorrectTemplate = df.IsCorrectTemplate.replace({"None": None, "True": True, "False": False})

    if df.HasTemplate.dtype != 'bool':
        df.HasTemplate = df.HasTemplate.replace({"None": None, "True": True, "False": False})

    # convert currency
    for col in ["Cost", "CostSearch", "CostRSYA", "CostSpec", "CostOther", "CostMobile"]:
        df[col] = df[col].replace('None', '0')
        df[col] = df[col].replace('', '0')
        df[col] = df[col].astype(pd.np.float64) * df.Currency

    df.drop(["Currency"], axis=1, inplace=True)

    df = filter_by_campaigns(logger, df)

    df["ShowsDesktop"] = df["Shows"] - df["ShowsMobile"]
    df["ClicksDesktop"] = df["Clicks"] - df["ClicksMobile"]
    df["CostDesktop"] = df["Cost"] - df["CostMobile"]

    return df


def LoginDF(logger):
    logger(logging.DEBUG, "start LoginDF", "app_debug")
    df_report = preprocess_report(logger, sources_from_tar(logger, "_keywords"))
    # Удалить костыль после релиза в прод CUSTOMSOLUTIONS-1958
    if df_report is None or df_report.shape[0] == 0:
        df_report = preprocess_report(logger, sources_from_tar(logger, "_report"))
    logger(logging.DEBUG, "finish LoginDF", "app_debug")

    return df_report


def SearchQueries(logger):
    logger(logging.DEBUG, "start SearchQueries", "app_debug")
    df_report = preprocess_search_queries(logger, sources_from_tar(logger, "_search_queries"))
    logger(logging.DEBUG, "finish SearchQueries", "app_debug")

    return df_report


def MobileDF(logger):
    logger(logging.DEBUG, "start MobileDF", "app_debug")
    df_report = preprocess_mobile(logger, sources_from_tar(logger, "_mobile"))
    logger(logging.DEBUG, "finish MobileDF", "app_debug")

    return df_report
