# coding: utf-8
"""utils"""
from .const import EXCEL_MAX_ROWS
import logging
try:
    import urllib2
except ImportError:
    import urllib.request as urllib2
import json
import re
from os.path import isdir
from os import mkdir
from distutils.version import LooseVersion

from PIL import Image
import pandas as pd


def extract_currency(file_name):
    "(str) -> float"
    if "_" in file_name:
        k = float(file_name.rsplit(".", 1)[0].rsplit("_", 1)[-1])
    else:
        k = 1.0

    return k


def extract_str_currency_from_tar(tar_path):
    """(str) -> str
    tar_path = "<login>_<currency>_<yyyymmdd>:<yyyymmdd>.tar.gz"
    """
    code = tar_path.split("_", 2)[1].strip()
    if ',' in code:
        raise ValueError("forbidden symbol in <%s>: %s" % (tar_path, ','))
    if not code:
        raise ValueError("there is no currency code: <%s>" % code)

    return code


def extract_currency_from_tar(tar_path):
    """(str) -> float
    tar_path = "<login>_<currency>_<yyyymmdd>:<yyyymmdd>.tar.gz"
    """
    currency_str = extract_str_currency_from_tar(tar_path)
    return float(currency_str)


def search_cid(s):

    return "https://direct.yandex.ru/registered/main.pl?searchcid=%s&cmd=search&who=camps" % s


def search_bid(s):

    return "https://direct.yandex.ru/registered/main.pl?cmd=searchBanners&where=direct&what=num&text_search=%s" % s


def init_app():
    """
    () -> DataFrame
    Read sources, create folders
    """
    if not isdir("./charts"):
        mkdir("./charts")
    if not isdir("./images"):
        mkdir("./images")
    if not isdir("./tmp_data"):
        mkdir("./tmp_data")


def fetch_last_remote_version(logger, pkg_name):
    """() -> unicode?"""
    last_releas = None
    URL = "https://pypi.python.org/pypi/{}/json".format(pkg_name)

    req = urllib2.Request(URL, headers={"Accept": "application/json"})
    try:
        res = urllib2.urlopen(req)
        if res.code == 200:
            releases = sorted(
                json.load(res)["releases"].keys(), key=LooseVersion)
            releases = filter(lambda x: "dev" not in x, releases)
            last_releas = releases[-1]
        res.close()
    except Exception as err:
        if logger:
            logger(logging.ERROR, err, "app_error", exc_info=1)

    return last_releas


def check_pkg_version(logger, pkg_name, cur_version):
    last_remote_version = fetch_last_remote_version(logger, pkg_name) or "0"
    if LooseVersion(cur_version) < LooseVersion(last_remote_version):
        msg = "Warning. You are using old version {}, new version {} is available.\n".format(
            cur_version, last_remote_version)
        print(
            colorize_str_to_console(msg, "red"), "Try upgrade via the:\n",
            "pip install --upgrade {}\n".format(pkg_name))


def get_image_shape(img_hash):
    """
    (str) -> typle<str, int?, int?>

    >>> get_image_shape("1ASndLYnISWIO68c5ksAUA")
    ('1ASndLYnISWIO68c5ksAUA', 173, 150)
    """
    url = "https://direct.yandex.ru/images/%s" % img_hash
    try:
        res = urllib2.urlopen(url)
        if res.code == 200:
            img = Image.open(res)
            height, width = img.size
            with open("./images/%s.png" % img_hash, "w") as fd:
                img.save(fd)
    except (IOError, ValueError):
        height, width = None, None

    return img_hash, height, width


def get_strategy_type(row):
    """
    (dict<str, float>) -> 'None' | 'only_rsya' | 'only_search' | 'both'

    expected keys: ShowsRSYA, ShowsSearch
    """

    has_rsya = row['ShowsRSYA'] > 0
    has_search = row['ShowsSearch'] > 0

    if has_rsya and has_search:
        return 'both'
    elif has_rsya:
        return 'only_rsya'
    elif has_search:
        return 'only_search'
    else:
        return 'None'


def extract_timetarget(s):
    """(str)->DataFrame
    #fixme try to use map(literal_eval, literal_eval(s)["Schedule"]["Items"])
    """
    return pd.DataFrame(
        map(lambda x: map(int, x.split(",")),
            re.findall("(\d[\d,]+\d)", s.split("Items", 1)[1].encode("u8"))),
        index=(u"Пн.", u"Вт.", u"Ср.", u"Чт.", u"Пт.", u"Сб.", u"Вс."))


def drop_antiwords(text):

    return re.split("( [-~][^ ]+)+",
                    text.strip())[0].strip()  # drop anti-words, ~0


def text_to_words(text):
    translate_table = dict((ord(char), u" ") for char in u"!?.,")
    text = text.translate(translate_table)
    text = re.split("[ ]+", text.strip())

    return text


def is_uppercase_word(text):
    """(unicode) -> bool"""
    if not isinstance(text, unicode):
        raise TypeError("expected unicode for `text`. in fact: %s" %
                        type(text))

    white_list = {u"РФ"}

    return (len(text) > 1) and all(map(unicode.isupper, text)) and (
        text not in white_list)


def has_uppercase_word(text):
    """(unicode) -> bool"""
    if not isinstance(text, unicode):
        raise TypeError("expected unicode for `text`. in fact: %s" %
                        type(text))

    words = text_to_words(text)

    return any(map(is_uppercase_word, words))


def phrase_process(text, to_lower=True):
    """(str|unicode) -> unicode"""
    try:
        text = text.strip("\"[]()")
    except AttributeError:
        return u""

    if not isinstance(text, unicode):
        text = text.decode("utf-8")

    if to_lower:
        text = text.lower()
    text = drop_antiwords(text)

    return " ".join(sorted(text_to_words(text)))


def classify_imgsize(shape):
    """
    (tuple<width:int, height:int>) -> 'bad_img (<= 450px)' | 'good_img (>= 1000)' | 'norm_img (others)'
    """
    w, h = shape
    if h <= 450 or w <= 450:
        return 'bad_img (<= 450px)'
    if h >= 1000 or w >= 1000:
        return 'good_img (>= 1000)'

    return 'norm_img (others)'


def note_maker(text, cell="A1", **kw_opt):
    """(str, str, **kw_opt) -> ((ExcelWriter, str, *args, **kw_args) -> ExcelWriter)"""

    def note(writer, sheet_name, *_, **__):
        sheet = writer.sheets[sheet_name]
        sheet.write_comment(cell, text, kw_opt)

        return writer

    return note


def add_sheet_to_xlsx(logger, writer, df, sheet_name, sheet_fmt, **kw_opt):
    """(ExcelWriter, DataFrame, str, [(ExcelWriter, str, DataFrame) -> ExcelWriter], **kw_opt) ->
       ExcelWriter
    Используя `ExcelWriter` сохраняет `DataFrame` на вкладку с именем `sheet_name`
    """

    if df is None or df.shape[0] == 0:
        print(colorize_str_to_console(sheet_name, "red"))
        logger(logging.ERROR, sheet_name, "app_add_sheet")

        return writer

    df.iloc[:EXCEL_MAX_ROWS].to_excel(
        writer, sheet_name, encoding="cp1251", **kw_opt)
    for fmt in sheet_fmt:
        fmt(writer, sheet_name, df)

    print(sheet_name)
    logger(logging.INFO, sheet_name, "app_add_sheet")

    return writer


def filter_by_adtype(df, adtype):

    assert adtype in {
        "TEXT_AD",
    }

    if "AdType" in df.columns:
        return df.loc[df["AdType"] == adtype]

    return df


def colorize_str_to_console(text, color):
    _CONSOLE_COLOR = {
        "default": "\033[0m",  # white (normal)
        "red": "\033[31m",
        "green": "\033[32m",
        "orange": "\033[33m",
        "blue": "\033[34m",
        "purple": "\033[35m",
    }
    if color not in _CONSOLE_COLOR:
        raise TypeError("`color` is not in %s" % _CONSOLE_COLOR.keys())

    options = {
        "color": _CONSOLE_COLOR[color],
        "text": text,
        "default": _CONSOLE_COLOR["default"]
    }
    template = "{color}{text}{default}"

    return template.format(**options)
