# coding: utf-8
"""xlsx-report:
  - table_total_campaigns_stat
  - 04_campaigns_type
  - 04_list_campaigns_type

Help:
[Раздельное управление площадками]
  (https://yandex.ru/support/direct/strategies/independent.xml?lang=en)
[Creating a separate campaign for displaying ads in the ad networks]
  (https://yandex.ru/support/direct/efficiency/content-websites.xml?lang=en)
"""
from ..xlsx_chart.chart import Chart, chart_add_series
from ..utils import search_cid, get_strategy_type
from ast import literal_eval
import seaborn as sns
import pandas as pd


def extract_strategy(cell, adv_type):
    """
    Returns value of "BiddingStrategyType" or "None"

    Params:
    cell :: str - encoded dict:
        u"{u'Search': {u'BiddingStrategyType': u'HIGHEST_POSITION'},
            u'Network': {u'BiddingStrategyType': u'SERVING_OFF'}}"
    adv_type :: "Search" | "Network"
    """
    assert adv_type in {"Search", "Network"}
    none_val = "None"

    try:
        strategy_opt = literal_eval(cell)
    except SyntaxError:
        return none_val

    return strategy_opt.get(adv_type, {"BiddingStrategyType": none_val}).get("BiddingStrategyType", none_val)


def table_total_campaigns_stat(df):
    """00_total_campaigns_stat"""

    df_chart = (df[["ShowsSpec", "ClicksSpec", "CostSpec",
                    "ShowsOther", "ClicksOther", "CostOther",
                    "ShowsRSYA", "ClicksRSYA", "CostRSYA",
                    "CampaignID", "CampaignName", "DayBudget", "Strategy"]]
                .groupby(["CampaignID", "CampaignName", "DayBudget", "Strategy"], as_index=False)
                .sum()
                .assign(SearchStrategy=lambda df: df.Strategy.apply(lambda x: extract_strategy(x, "Search")))
                .assign(RSYAStrategy=lambda df: df.Strategy.apply(lambda x: extract_strategy(x, "Network")))
                .assign(HelperUrl=lambda df: df.CampaignID.apply(search_cid))
                .drop("Strategy", axis=1))

    chart_total_campaigns_stat(df_chart)

    df_chart = df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "CampaignName": u"Название кампании",
        "DayBudget": u"Дневной бюджет",
        "ShowsSpec": u"Показы (спец)",
        "ClicksSpec": u"Клики (спец)",
        "CostSpec": u"Затраты (спец)",
        "ShowsOther": u"Показы (юж./прав. блок)",
        "ClicksOther": u"Клики (юж./прав. блок)",
        "CostOther": u"Затраты (юж./прав. блок)",
        "ShowsRSYA": u"Показы (сети)",
        "ClicksRSYA": u"Клики (сети)",
        "CostRSYA": u"Затраты (сети)",
        "SearchStrategy": u"Стратегия на поиске",
        "RSYAStrategy": u"Стратегия в сетях",
        "HelperUrl": u"Ссылка на кампанию"})

    return df_chart


def chart_total_campaigns_stat(df_chart):
    """00_campaign_strat_type"""
    sns.plt.figure()
    sns.plt.rcParams.update({'figure.autolayout': True})

    ax = (df_chart.rename(columns={
        "SearchStrategy": u"Стратегия на поиске",
        "RSYAStrategy": u"Стратегия в сетях",
        "ShowsRSYA": u"Показы (сети)",
        "ClicksRSYA": u"Клики (сети)",
        "CostRSYA": u"Затраты (сети)",
        "CampaignID": u"Кол-во кампаний"})
        .groupby([u"Стратегия на поиске", u"Стратегия в сетях"])
        .agg({
            u"Показы (сети)": pd.np.sum,
            u"Клики (сети)": pd.np.sum,
            u"Затраты (сети)": pd.np.sum,
            u"Кол-во кампаний": lambda x: x.unique().size})
        .apply(lambda x: x / x.sum(), axis=0)
        .sort_index()
        .plot.barh())
    ax.set_title(u"Обзор стратегий", size=20)
    ax.set_xlabel(u"Доля от общего", size=16)
    ax.set_ylabel(u"(Стратегия на поиске, Стратегия в сетях)", size=16)
    sns.plt.savefig("./charts/00_campaign_strat_type.png", dpi=200)
    sns.plt.rcParams.update({'figure.autolayout': False})


def table_campaigns_type(df):
    """04_campaigns_type"""

    df_chart = (df[["ShowsSearch", "ShowsRSYA", "Shows", "Clicks", "Cost", "CampaignID"]]
                .groupby("CampaignID", as_index=False)
                .agg({"ShowsSearch": pd.np.sum,
                      "ShowsRSYA": pd.np.sum,
                      "Shows": pd.np.sum,
                      "Clicks": pd.np.sum,
                      "Cost": pd.np.sum}))
    df_chart['StrategyType'] = df_chart.apply(get_strategy_type, axis=1)
    df_chart = (df_chart[["Shows", "Clicks", "Cost", "CampaignID", "StrategyType"]]
                .groupby("StrategyType")
                .agg({"Shows": pd.np.sum,
                      "Clicks": pd.np.sum,
                      "Cost": pd.np.sum,
                      "CampaignID": lambda x: x.unique().size}))
    df_chart.rename(columns={
        "CampaignID": u"Кол-во кампаний",
        "Shows": u"Показы",
        "Clicks": u"Клики",
        "Cost": u"Затраты"},
        inplace=True)
    df_chart.rename(index={
        "only_search": u"Только поиск",
        "only_rsya": u"Только сети",
        "both": u"Все площадки"},
        inplace=True)
    df_chart.index.name = u"Стратегия показа"
    if df_chart.shape[0] == 0:
        return None
    return df_chart.loc[[u"Все площадки", u"Только поиск", u"Только сети"]].fillna('')


def plot_campaigns_type(writer, sheet_name, df):
    """(ExcelWriter, str, DataFrame) -> ExcelWriter"""
    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "bar", "percent_stacked", u'Разделение по площадкам')
    for x in range(3):
        irow = 2 + x
        chart_add_series(chart, x, {
            'values': '={0}!B{1}:E{1}'.format(sheet_name, irow),
            "categories": '={}!B1:E1'.format(sheet_name),
            "name": "={0}!A{1}".format(sheet_name, irow)})

    sheet.insert_chart('A6', chart)

    return writer


def table_list_campaigns_type(df):
    """04_list_campaigns_type"""

    df_chart = (df[["CampaignID", "Cost",
                    "ShowsSearch", "ClicksSearch", "CostSearch",
                    "ShowsRSYA", "ClicksRSYA", "CostRSYA"]]
                .groupby("CampaignID", as_index=False)
                .agg({"Cost": pd.np.sum,
                      "ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "ShowsRSYA": pd.np.sum,
                      "ClicksRSYA": pd.np.sum,
                      "CostRSYA": pd.np.sum})
                .assign(StrategyType=lambda x: x.apply(get_strategy_type, axis=1))
                .sort_values("Cost", ascending=False)
                .reset_index(drop=True)
                [["CampaignID", "StrategyType",
                  "ShowsSearch", "ClicksSearch", "CostSearch",
                  "ShowsRSYA", "ClicksRSYA", "CostRSYA"]])
    df_chart.StrategyType = df_chart.StrategyType.replace(to_replace={
        "only_search": u"Только поиск",
        "only_rsya": u"Только сети",
        "both": u"Все площадки"})
    df_chart.rename(columns={
        "CampaignID": u"Кампания №",
        "StrategyType": u"Стратегия показа",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)",
        "ShowsRSYA": u"Показы (сети)",
        "ClicksRSYA": u"Клики (сети)",
        "CostRSYA": u"Затраты (сети)"},
        inplace=True)

    return df_chart
