# coding: utf-8
"""xlsx-report:
  - 01_highlight_hist
  - 01_highlight_type
  - 01_campaign_highlight_type
"""
import pandas as pd
from ..xlsx_chart.chart import Chart, chart_add_series, FONT_NAME
from ..utils import filter_by_adtype
# import seaborn as sns


def table_highlight_type(df):
    """01 Highlight (Search)"""

    def overlap_type(overlap):

        if overlap >= .99:
            return u'full'

        if overlap <= .01:
            return u'none'

        return u'partly'

    df_chart = filter_by_adtype(df, "TEXT_AD")
    df_chart = df_chart[[
        "BoldedWordsCnt",
        "WordsCnt",
        "ShowsSearch",
        "ClicksSearch",
        "CostSearch",
        "BSPhraseID",
        "CampaignID",
        "DirectBannerID"]].copy()
    df_chart["PhraseBoldShare"] = df_chart.BoldedWordsCnt / df_chart.WordsCnt
    df_chart['PhraseBoldType'] = df_chart.PhraseBoldShare.apply(overlap_type)

    return df_chart


def table_highlight_hist(df, normalize):
    """01_highlight_hist"""

    df_chart = table_highlight_type(df)
    df_chart = df_chart[(df_chart.ShowsSearch > 0)]
    if df_chart.shape[0] == 0:
        return None

    df_chart.PhraseBoldShare = (df_chart.PhraseBoldShare * 100).round(-1).astype(int).values
    df_chart = (df_chart[["DirectBannerID", "BSPhraseID", "ShowsSearch", "ClicksSearch", "CostSearch", "PhraseBoldShare"]]
                .groupby(["PhraseBoldShare"], as_index=True)
                .agg({"ShowsSearch": pd.np.sum,
                      "ClicksSearch": pd.np.sum,
                      "CostSearch": pd.np.sum,
                      "BSPhraseID": lambda x: x.unique().size,
                      "DirectBannerID": lambda x: x.unique().size}))

    df_chart.rename(columns={
        "BSPhraseID": u"Кол-во фраз",
        "DirectBannerID": u"Кол-во баннеров",
        "ShowsSearch": u"Показы (Поиск)",
        "ClicksSearch": u"Клики (Поиск)",
        "CostSearch": u"Затраты (Поиск)"},
        inplace=True)
    df_chart.index.names = [u"Подсветка"]
    df_chart = df_chart.loc[range(0, 110, 10)].fillna(pd.np.nan)

    if normalize:
        df_chart = df_chart.apply(lambda x: x / x.sum())

    return df_chart[[u"Кол-во баннеров", u"Кол-во фраз", u"Показы (Поиск)", u"Клики (Поиск)", u"Затраты (Поиск)"]]


def plot_highlight_hist(writer, sheet_name, df):
    """
    (ExcelWriter, str, DataFrame) -> ExcelWriter
    01_highlight_hist
    """

    sheet = writer.sheets[sheet_name]
    chart = Chart(writer.book, "column", None, u'Подсветка рекламных фраз')
    chart.set_y_axis({"name": u"Доля от общего",
                      'name_font': {"size": 16, "bold": False, "name": FONT_NAME},
                      'num_font': {"size": 14, "bold": False, "name": FONT_NAME},
                      'num_format': '#0%',
                      'major_tick_mark': 'none',
                      'minor_tick_mark': 'none',
                      'line': {'none': True}})
    cols = "BCDEF"
    last_row = df.index.size + 2
    for n, col in enumerate(cols):
        chart_add_series(chart, n, {
            'values': '={0}!{1}2:{1}{2}'.format(sheet_name, col, last_row),
            "categories": '={}!A2:A{}'.format(sheet_name, last_row),
            "name": "={0}!{1}1".format(sheet_name, col),
            "data_labels": {"value": False}})

    sheet.insert_chart('A%s' % (last_row + 2), chart)

    return writer


def table_highlight_type_phrases(df):
    "01_highlight_type"

    df_export = table_highlight_type(df)
    df_export = df_export[df_export.ShowsSearch > 0]
    if df_export.shape[0] == 0:
        return None

    df_export = (df_export[['BSPhraseID', 'PhraseBoldType', "ShowsSearch", "ClicksSearch", "CostSearch"]]
                    .groupby('PhraseBoldType')
                    .agg({'ShowsSearch': pd.np.sum,
                          'ClicksSearch': pd.np.sum,
                          'CostSearch': pd.np.sum,
                          'BSPhraseID': pd.np.count_nonzero}))
    df_export.rename(columns={
        "BSPhraseID": u"Количество фраз",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_export.rename(index={
        "full": u"Полная",
        "partly": u"Частичная",
        "none": u"Отсутствует"},
        inplace=True)
    df_export.index.name = u"Тип подсветки"

    return df_export.loc[[u"Полная", u"Частичная", u"Отсутствует"]].fillna('')


def plot_highlight_type_phrases(writer, sheet_name, df):
    """
    (ExcelWriter, str, DataFrame) -> ExcelWriter
    01_highlight_type
    """

    sheet = writer.sheets[sheet_name]

    # Create chart
    chart = Chart(writer.book, "bar", "percent_stacked", u'Подсветка фраз')

    # Add data series.
    chart_add_series(chart, 0, {
        'values': '=%s!B2:E2' % sheet_name,
        "categories": '=%s!B1:E1' % sheet_name,
        "name": "=%s!A2" % sheet_name})
    chart_add_series(chart, 1, {
        'values': '=%s!B3:E3' % sheet_name,
        "categories": '=%s!B1:E1' % sheet_name,
        "name": "=%s!A3" % sheet_name})
    chart_add_series(chart, 2, {
        'values': '=%s!B4:E4' % sheet_name,
        "categories": '=%s!B1:E1' % sheet_name,
        "name": "=%s!A4" % sheet_name})

    # Insert the chart into the worksheet.
    sheet.insert_chart('A7', chart)

    return writer


def table_highlight_type_campaign(df):
    "01_campaign_highlight_type"

    df_export = table_highlight_type(df)
    df_export = df_export[df_export.ShowsSearch > 0]
    if df_export.shape[0] == 0:
        return None

    df_export = (df_export[['CampaignID', 'BSPhraseID', 'PhraseBoldType', "ShowsSearch", "ClicksSearch", "CostSearch"]]
                    .groupby(['CampaignID', 'PhraseBoldType'], as_index=True)
                    .agg({'ShowsSearch': pd.np.sum,
                          'ClicksSearch': pd.np.sum,
                          'CostSearch': pd.np.sum,
                          'BSPhraseID': pd.np.count_nonzero}))
    df_export.rename(columns={
        "BSPhraseID": u"Количество фраз",
        "ShowsSearch": u"Показы (поиск)",
        "ClicksSearch": u"Клики (поиск)",
        "CostSearch": u"Затраты (поиск)"},
        inplace=True)
    df_export.rename(index={
        "full": u"Полная",
        "partly": u"Частичная",
        "none": u"Отсуствует"},
        inplace=True)
    df_export.index.names = [u"Кампания №", u"Тип подсветки"]

    return df_export
